"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskLogViewer = void 0;
const react_1 = __importDefault(require("react"));
const date_fns_1 = require("date-fns");
const polaris_1 = require("@shopify/polaris");
const taskify_shared_1 = require("@dotdev/taskify-shared");
const polaris_icons_1 = require("@shopify/polaris-icons");
const hooks_1 = require("../hooks");
const TaskLogIcon = ({ taskLog }) => {
    if (taskify_shared_1.LogLevel.ERROR === taskLog.logLevel)
        return react_1.default.createElement(polaris_1.Icon, { source: polaris_icons_1.AlertMinor, color: "critical" });
    if (taskify_shared_1.LogLevel.WARN === taskLog.logLevel)
        return react_1.default.createElement(polaris_1.Icon, { source: polaris_icons_1.RiskMinor, color: "warning" });
    return react_1.default.createElement(polaris_1.Icon, { source: polaris_icons_1.ChevronRightMinor, color: "subdued" });
};
const TaskLogMessage = ({ taskLog }) => {
    if (taskify_shared_1.LogLevel.ERROR === taskLog.logLevel)
        return (react_1.default.createElement("p", null,
            react_1.default.createElement(polaris_1.TextStyle, { variation: "negative" }, taskLog.message)));
    if (taskify_shared_1.LogLevel.DEBUG === taskLog.logLevel)
        return (react_1.default.createElement("p", null,
            react_1.default.createElement(polaris_1.TextStyle, { variation: "subdued" }, taskLog.message)));
    return react_1.default.createElement("p", null, taskLog.message);
};
const TaskLogViewer = ({ taskAttemptRef }) => {
    const { taskLogs } = hooks_1.useTaskLogs(taskAttemptRef);
    return (react_1.default.createElement("div", { className: "taskify-log-viewer" },
        !!taskAttemptRef && !taskLogs && (react_1.default.createElement(polaris_1.Stack, null,
            react_1.default.createElement(polaris_1.Spinner, { size: "small" }),
            react_1.default.createElement("p", null, "Loading logs\u2026"))),
        !!taskLogs && 0 === taskLogs.length && (react_1.default.createElement(polaris_1.Stack, null,
            react_1.default.createElement(polaris_1.Spinner, { size: "small" }),
            react_1.default.createElement("p", null, "Waiting for logs\u2026"))),
        !!taskLogs && 0 < taskLogs.length && (react_1.default.createElement(polaris_1.Stack, { spacing: "tight", vertical: true }, taskLogs.map((taskLog) => (react_1.default.createElement(polaris_1.Stack, { wrap: false, spacing: "tight", key: taskLog.id, alignment: "leading" },
            react_1.default.createElement("p", { className: "taskify-log-viewer__time" }, date_fns_1.format(taskLog.createdAt.toDate(), "HH:mm:ss")),
            react_1.default.createElement("div", { className: "taskify-log-viewer__icon" },
                react_1.default.createElement(TaskLogIcon, { taskLog: taskLog })),
            react_1.default.createElement(polaris_1.Stack.Item, { fill: true },
                react_1.default.createElement(TaskLogMessage, { taskLog: taskLog })))))))));
};
exports.TaskLogViewer = TaskLogViewer;
