"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useViewTaskPath = exports.useViewTask = exports.useRetryTask = exports.useRunTask = exports.useTaskLogs = exports.useTaskAttempts = exports.useTask = exports.useTasks = void 0;
const react_1 = __importDefault(require("react"));
const firestore_1 = require("react-firebase-hooks/firestore");
const taskify_shared_1 = require("@dotdev/taskify-shared");
const provider_1 = require("./provider");
const firebase = () => window.firebase;
const serverTimestamp = () => firebase().firestore.FieldValue.serverTimestamp();
const useTasks = (options = {}) => {
    const limit = react_1.default.useMemo(() => { var _a; return (_a = options.limit) !== null && _a !== void 0 ? _a : 20; }, [options]);
    const [startAfters, setStartAfters] = react_1.default.useState([]);
    const query = react_1.default.useMemo(() => {
        let query = firebase().firestore().collectionGroup("tasks");
        if (options.taskGroup) {
            query = query.where("taskGroup", "==", options.taskGroup);
        }
        if (options.taskType) {
            query = query.where("taskType", "==", options.taskType);
        }
        if (options.taskTypes && 0 < options.taskTypes.length) {
            query = query.where("taskType", "in", options.taskTypes);
        }
        if (options.status) {
            query = query.where("status", "==", options.status);
        }
        return query
            .orderBy("createdAt", "desc")
            .limit(limit)
            .startAfter(0 < startAfters.length ? startAfters[startAfters.length - 1] : []);
    }, [options]);
    const [dataItems, loading, error] = firestore_1.useCollectionData(query, { refField: "ref", idField: "id" });
    // log errors to console
    react_1.default.useEffect(() => error && console.error(error), [error]);
    // keep items in local state to stop them getting set to undefined during pagination
    // which causes the UI to disappear
    const [items, setItems] = react_1.default.useState(dataItems);
    // // clear items when the options has changed
    // React.useEffect(() => {
    //   setItems(undefined);
    //   setStartAfters([]);
    // }, [options]);
    // update local state when query returns
    react_1.default.useEffect(() => {
        if (dataItems)
            setItems(dataItems);
    }, [dataItems]);
    const hasNextPage = react_1.default.useMemo(() => !!items && limit === items.length, [items]);
    const nextPage = react_1.default.useCallback(() => {
        if (!hasNextPage || !items)
            return;
        setStartAfters((startAfters) => [
            ...startAfters,
            items[items.length - 1].createdAt,
        ]);
    }, [hasNextPage, items]);
    const hasPrevPage = react_1.default.useMemo(() => !!items && 0 < startAfters.length, [items, startAfters]);
    const prevPage = react_1.default.useCallback(() => {
        if (!hasPrevPage || !items)
            return;
        setStartAfters((startAfters) => startAfters.slice(0, -1));
    }, [hasPrevPage, items]);
    return {
        tasks: items,
        loading,
        error,
        nextPage,
        hasNextPage,
        prevPage,
        hasPrevPage,
    };
};
exports.useTasks = useTasks;
const useTask = (taskRef) => {
    const [task, loading, error] = firestore_1.useDocumentData(taskRef !== null && taskRef !== void 0 ? taskRef : undefined, { refField: "ref", idField: "id" });
    // log errors to console
    react_1.default.useEffect(() => error && console.error(error), [error]);
    return { task, loading, error };
};
exports.useTask = useTask;
const useTaskAttempts = (taskRef) => {
    const [taskAttempts, loading, error] = firestore_1.useCollectionData(taskRef
        ? taskRef.collection("taskAttempts").orderBy("createdAt", "desc")
        : undefined, { refField: "ref", idField: "id" });
    // log errors to console
    react_1.default.useEffect(() => error && console.error(error), [error]);
    return {
        taskAttempts,
        loading,
        error,
    };
};
exports.useTaskAttempts = useTaskAttempts;
const useTaskLogs = (taskAttemptRef) => {
    const [taskLogs, loading, error] = firestore_1.useCollectionData(taskAttemptRef
        ? taskAttemptRef.collection("logs").orderBy("createdAt", "desc")
        : undefined, { refField: "ref", idField: "id" });
    return {
        taskLogs,
        loading,
        error,
    };
};
exports.useTaskLogs = useTaskLogs;
const useRunTask = () => {
    const viewTask = exports.useViewTask();
    return react_1.default.useCallback((taskRequest, options = {}) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const openModal = (_a = options.openModal) !== null && _a !== void 0 ? _a : true;
        const task = Object.assign(Object.assign({}, taskRequest), { status: taskify_shared_1.TaskStatus.PENDING, createdAt: serverTimestamp() });
        const taskRef = (yield firebase()
            .firestore()
            .collection([
            "taskGroups",
            task.taskGroup,
            "taskTypes",
            task.taskType,
            "tasks",
        ].join("/"))
            .add(task));
        const taskAttempt = {
            status: taskify_shared_1.TaskStatus.PENDING,
            createdAt: serverTimestamp(),
        };
        const taskAttemptRef = (yield taskRef
            .collection("taskAttempts")
            .add(taskAttempt));
        const taskInvocation = {
            invocationCount: 1,
            status: taskify_shared_1.TaskStatus.PENDING,
            createdAt: serverTimestamp(),
        };
        const taskInvocationRef = (yield taskAttemptRef
            .collection("taskInvocations")
            .add(taskInvocation));
        if (openModal)
            viewTask(taskRef);
        return { taskRef, taskAttemptRef, taskInvocationRef };
    }), []);
};
exports.useRunTask = useRunTask;
const useRetryTask = (taskRef) => react_1.default.useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
    if (!taskRef)
        return;
    const task = (yield taskRef.get()).data();
    if (!task)
        return;
    if (![taskify_shared_1.TaskStatus.FAILED, taskify_shared_1.TaskStatus.COMPLETE, taskify_shared_1.TaskStatus.TIMEOUT].includes(task.status)) {
        throw new Error(`cannot retry a task in ${task.status} state`);
    }
    const taskAttempt = {
        status: taskify_shared_1.TaskStatus.PENDING,
        createdAt: serverTimestamp(),
    };
    const taskAttemptRef = (yield taskRef
        .collection("taskAttempts")
        .add(taskAttempt));
    const taskInvocation = {
        invocationCount: 1,
        status: taskify_shared_1.TaskStatus.PENDING,
        createdAt: serverTimestamp(),
    };
    const taskInvocationRef = (yield taskAttemptRef
        .collection("taskInvocations")
        .add(taskInvocation));
    return { taskAttemptRef, taskInvocationRef };
}), [taskRef]);
exports.useRetryTask = useRetryTask;
const useViewTask = () => provider_1.useContext().viewTask;
exports.useViewTask = useViewTask;
const useViewTaskPath = () => {
    const viewTask = exports.useViewTask();
    return react_1.default.useCallback((taskPath) => {
        const taskRef = firebase()
            .firestore()
            .doc(taskPath);
        return viewTask(taskRef);
    }, [viewTask]);
};
exports.useViewTaskPath = useViewTaskPath;
